import { Injectable, CanActivate, ExecutionContext, UnauthorizedException, ForbiddenException } from '@nestjs/common';
import { Reflector } from '@nestjs/core';
import { SessionsService } from '../services/sessions.service';

export class PermissionGuardArgs {
    keywords?: string[] | null | undefined;
    verificationLevels?: string[] | null | undefined;
}

export const AllowPermissions = Reflector.createDecorator<PermissionGuardArgs>();

@Injectable()
export class PermissionGuard implements CanActivate {
    constructor(private readonly _sessionsService: SessionsService, private readonly _reflector: Reflector) {}

    async canActivate(context: ExecutionContext) {
        const args = this._reflector.get<PermissionGuardArgs>(AllowPermissions, context.getHandler());

        const request = context.switchToHttp().getRequest();
        const token: string = this._getToken(request);

        if (!token) {
            throw new UnauthorizedException();
        }

        const session = await this._sessionsService.read(token);
        if (!session) {
            throw new UnauthorizedException();
        }

        let isPermissionsOk = true;
        if (args && args.keywords) {
            isPermissionsOk = false;
            for (let keyword of args.keywords) {
                isPermissionsOk = this._checkPermission(session.user.permissions, keyword);
                if (isPermissionsOk) {
                    break;
                }
            }
        }

        if (!isPermissionsOk) {
            throw new ForbiddenException();
        }

        request.session = session;
        return true;
    }

    private _getToken(request: any) {
        let token = request?.headers?.authorization;

        if (!token) {
            return null;
        }

        token = token.replace('bearer ', '');
        token = token.replace('Bearer ', '');

        return token;
    }

    private _checkPermission(userPermissions: string[], allowedPermission: string): boolean {
        return userPermissions.some((keyword) => keyword === allowedPermission);
    }
}
